@@ -2,46 +2,51 @@ program sound_conv
 ! ǤդΥǥ󥰥ǡᥤץबɤ߹ߤ䤹Ѵ.
   use file_operate
   use Basis
+  use Math_Const
   use Thermo_Const
   use Thermo_Function
+  use read_name
 
   implicit none
 
   real, allocatable, dimension(:,:) :: val
   character(30), allocatable, dimension(:,:) :: cval
-  integer :: i, j, k, nx, nz, i_undef, skip_num, rh_flag, nf
-  character(30) :: flag, sign_flag
-  character(80) :: list_name, output_name, undef, fname
-  character(80), allocatable, dimension(:) :: iname, oname
-  character(10) :: unity(4)  ! unit convert flags
+  integer :: id, i, j, k, nx, nz, i_undef, skip_num, rh_flag, nf
+  character(100) :: sign_flag, tmpc
+  character(200) :: output_name, undef, fname
+  character(200), allocatable, dimension(:) :: iname, oname
+  character(10) :: unity(5)  ! unit convert flags
   real, parameter :: conv_undef=-999.0
-  real :: limit_height, factor
+  real :: limit_height, factor, ew_tmp, ww_tmp
   real, allocatable, dimension(:) :: height, pres, temp, vapor, tmp, ew, ww
 
   i_undef=len_trim(undef)
 
 !-- namelist file read
 
-  namelist/ input/ flag, fname, undef, skip_num, unity, limit_height, output_name
+  call read_namelist()
+  namelist /input /fname, undef, sign_flag, skip_num, unity, limit_height, output_name
   read(5,nml=input)
 
 !-- list file read
 
-  nf=line_number_counter( trim(list_name) )
+  nf=line_number_counter( trim(fname) )
   allocate(iname(nf))
   allocate(oname(nf))
-  call read_file_text( trim(list_name), 1, nf, iname )
+  call read_file_text( trim(fname), 1, nf, iname )
 
 !-- do loop counter
 
-  do i=1,nf
+  do id=1,nf
+
+     write(*,*) "--- starting to convert Now: ", trim(iname(id)), " ---"
 
 !-- column and array set
 
-     nx=len_trim(flag)
-     nz=line_number_counter( trim(iname(i)) )
+     nx=len_trim(sign_flag)
+     nz=line_number_counter( trim(iname(id)) )-skip_num
 
-     oname(i)="conv-"//iname(i)
+     oname(id)="conv-"//iname(id)
 
      allocate(cval(nx,nz))
      allocate(val(nx,nz))
@@ -55,14 +60,15 @@ program sound_conv
 
 !-- reading file
 
-     call read_file_text( trim(iname(i)), nx, nz, cval, skip=skip_num )
+     call read_file_text( trim(iname(id)), nx, nz, cval, skip=skip_num )
 
 !-- type convert
 
      do j=1,nx
-        if(flag(j:j)=='o')then
+        if(sign_flag(j:j)/='0')then
            do k=1,nz
-              if(trim(cval(j,k))==trim(undef))then
+              tmpc=trim(cval(j,k))
+              if(tmpc(1:len_trim(undef))==trim(undef))then
                  val(j,k)=conv_undef
               else
                  val(j,k)=c2r_convert( cval(j,k) )
@@ -170,19 +176,62 @@ program sound_conv
         end do
      end if
 
+!-- convert check wind radian
+
+     if(unity(5)=='rad')then
+        do j=1,nz
+           if(vapor(j)/=conv_undef)then
+              ew_tmp=ew(j)
+              ww_tmp=ww(j)
+              ew(j)=ew_tmp*sin(ww_tmp)
+              ww(j)=ew_tmp*cos(ww_tmp)
+           end if
+        end do
+     end if
+
+     if(unity(5)=='degree')then
+        do j=1,nz
+           if(ew(j)/=conv_undef.or.ww(j)/=conv_undef)then
+              ew_tmp=ew(j)
+              ww_tmp=ww(j)*pi/180.0
+              ew(j)=ew_tmp*sin(ww_tmp)
+              ww(j)=ew_tmp*cos(ww_tmp)
+           end if
+        end do
+     end if
+
 !-- writing file
-     open(unit=11+2*i,file=trim(oname(i)),status='unknown')
-     write(11+i,*) 'height', 'pressure', 'temperature', 'vapor', 'eastwind', 'westwind'
-     write(11+i,*) "'m'", "'Pa'", "'K'", "'kg/kg'", "'m/s'", "'m/s'"
-     do j=1,nz
+
+     write(*,*) "--- writing Now: ", trim(oname(id)), " ---"
+
+     open(unit=11+id,file=trim(oname(id)),status='unknown')
+     write(11+id,*) 'height ', 'pressure ', 'temperature ', 'vapor ', 'E-Wwind ', 'N-Swind '
+     write(11+id,*) " 'm' ", " 'Pa' ", " 'K' ", " 'kg/kg' ", " 'm/s' ", " 'm/s'"
+     do j=1,nz-1
         if(height(j)>limit_height)then
-           write(11+i,'(1000f)') height(j), pres(j), temp(j), vapor(j), ew(j), ww(j)
+           write(11+id,'(1000f)') height(j), pres(j), temp(j), vapor(j), ew(j), ww(j)
            if(height(j+1)<height(j))then
               exit
            end if
         end if
+        if(j==nz-1)then
+           write(11+id,'(1000f)') height(j+1), pres(j+1), temp(j+1),  &
+  &                               vapor(j+1), ew(j+1), ww(j+1)
+        end if
      end do
-     close(unit=11+i,status='keep')
+     close(unit=11+id,status='keep')
+
+     deallocate(cval)
+     deallocate(val)
+     deallocate(height)
+     deallocate(temp)
+     deallocate(pres)
+     deallocate(vapor)
+     deallocate(ew)
+     deallocate(ww)
+     deallocate(tmp)
+
+     write(*,*) "--- finishing to convert Now: ", trim(iname(id)), " ---"
 
   end do
 
