@@ -1,8 +1,9 @@
 file = Array[ 
 "alge_solv.f90",
 "algebra.f90",
-"analy.f90",
+"assimilation.f90",
 "basis.f90",
+"derivation.f90",
 "ffts.f90",
 "file_operate.f90",
 "geometry.f90",
@@ -26,8 +27,9 @@ file = Array[
 dir = Array[ 
 "Alge_Solv",
 "Algebra",
-"analy",
+"Assimilation.f90",
 "Basis",
+"Derivation",
 "ffts",
 "file_operate",
 "Geometry",
