@@ -2,6 +2,13 @@ module Statistics  ! ײϴطΥ롼
 
   private :: summ
 
+interface Bubble_Sort
+! Bubble_Sort
+
+  module procedure Bubble_Sort_i, Bubble_Sort_f
+
+end interface Bubble_Sort
+
 contains
 
 subroutine Mean_1d( x, ave, error )  ! 1 ʿͷ׻롼
@@ -315,7 +322,7 @@ subroutine covariance( x, y, cov, error )  ! 2 Ĥ 1 ǡζʬ׻
 end subroutine covariance
 
 
-subroutine nearest_search_1d( x, point, i, undeff )
+subroutine nearest_search_1d( x, point, i )
   ! 1 Ƕ˵õ롼
   ! interpo_search_1d ͤ, ͤ +1 ͤεΥӤ
   ! Υû򤹤.
@@ -323,42 +330,38 @@ subroutine nearest_search_1d( x, point, i, undeff )
   real, intent(in) :: x(:)  ! 
   real, intent(in) :: point  ! 
   integer, intent(inout) :: i  ! point κǶ˵ֹ
-  real, intent(in), optional :: undeff  ! õϰϤǤ꾮ͤõ褦Ȥ, undef ֤,  undef ͤꤹ. default Ǥ 0.
   real :: tmp1, tmp2
-  integer :: j
+  integer :: j, nx
 
-  if(present(undeff))then
-     call interpo_search_1d( x, point, j, undeff )
+  nx=size(x)
+
+  call interpo_search_1d( x, point, j )
+
+  if(j==0)then  ! i=1 ˤΤ, tmp1 x(1), tmp2  x(2) ,  if ʸ
+           ! Ǥޤ.
+     tmp1=x(j+1)
+     tmp2=x(j+2)
   else
-     call interpo_search_1d( x, point, j )
+     if(j==nx)then  ! i=nx ˤΤ, tmp2  x(nx), tmp1  x(nx-1) ,
+            !  if ʸǤޤ.
+        tmp1=x(j)
+        tmp2=x(j-1)
+     else
+        tmp1=x(j)
+        tmp2=x(j+1)
+     end if
   end if
 
-  tmp1=x(j)
-  tmp2=x(j+1)
-
-  if(abs(point-tmp1)>=abs(tmp2-point))then
-     if(size(x)>=j+1)then  ! j+1 󻲾ȳξν
-        i=j+1
-     else
-        i=j
-     end if
+  if(abs(point-tmp1)>abs(tmp2-point))then
+     i=j+1
   else
      i=j
   end if
 
-  if(j==0)then
-     write(*,*) "WARNING: j is undef value."
-  end if
-  if(present(undeff))then
-     if(j==int(undeff))then
-        write(*,*) "WARNING: j is undef value."
-     end if
-  end if
-
 end subroutine nearest_search_1d
 
 
-subroutine nearest_search_2d( x, y, pointx, pointy, i, j, undeff )
+subroutine nearest_search_2d( x, y, pointx, pointy, i, j )
   ! 2 Ƕ˵õ롼
   ! nearest_search_1d ͤ.
   ! , 2 Ǥ뤿,  4 κǶ׻ɬפ뤬,
@@ -371,20 +374,14 @@ subroutine nearest_search_2d( x, y, pointx, pointy, i, j, undeff )
   real, intent(in) :: pointy  !  y
   integer, intent(inout) :: i  ! pointx κǶֹ
   integer, intent(inout) :: j  ! pointy κǶֹ
-  real, intent(in), optional :: undeff  ! õϰϤǤ꾮ͤõ褦Ȥ, undef ֤,  undef ͤꤹ. default Ǥ 0.
 
-  if(present(undeff))then
-     call nearest_search_1d( x, pointx, i, undeff )
-     call nearest_search_1d( y, pointy, j, undeff )
-  else
-     call nearest_search_1d( x, pointx, i )
-     call nearest_search_1d( y, pointy, j )
-  end if
+  call nearest_search_1d( x, pointx, i )
+  call nearest_search_1d( y, pointy, j )
 
 end subroutine nearest_search_2d
 
 
-subroutine nearest_search_3d( x, y, z, pointx, pointy, pointz, i, j, k, undeff )
+subroutine nearest_search_3d( x, y, z, pointx, pointy, pointz, i, j, k )
   ! 2 Ƕ˵õ롼
   ! nearest_search_1d ͤ.
   ! , 2 Ǥ뤿,  4 κǶ׻ɬפ뤬,
@@ -400,18 +397,11 @@ subroutine nearest_search_3d( x, y, z, pointx, pointy, pointz, i, j, k, undeff )
   integer, intent(inout) :: i  ! pointx κǶֹ
   integer, intent(inout) :: j  ! pointy κǶֹ
   integer, intent(inout) :: k  ! pointz κǶֹ
-  real, intent(in), optional :: undeff  ! õϰϤǤ꾮ͤõ褦Ȥ, undef ֤,  undef ͤꤹ. default Ǥ 0.
   integer :: just
 
-  if(present(undeff))then
-     call nearest_search_1d( x, pointx, i, undeff )
-     call nearest_search_1d( y, pointy, j, undeff )
-     call nearest_search_1d( z, pointz, k, undeff )
-  else
-     call nearest_search_1d( x, pointx, i )
-     call nearest_search_1d( y, pointy, j )
-     call nearest_search_1d( z, pointz, k )
-  end if
+  call nearest_search_1d( x, pointx, i )
+  call nearest_search_1d( y, pointy, j )
+  call nearest_search_1d( z, pointz, k )
 
 end subroutine nearest_search_3d
 
@@ -423,7 +413,7 @@ subroutine interpo_search_1d( x, point, i, undeff )
   real, intent(in) :: x(:)  ! 
   real, intent(in) :: point  ! 
   integer, intent(inout) :: i  ! point ͤۤʤֹͤ
-  real, intent(in), optional :: undeff  ! õϰϤǤ꾮ͤõ褦Ȥ, undef ֤,  undef ͤꤹ. default Ǥ 0.
+  integer, intent(in), optional :: undeff  ! õϰϤǤ꾮ͤõ褦Ȥ, undef ֤,  undef ͤꤹ. default Ǥ 0.
   integer :: nx, j
   integer :: just
 
@@ -473,13 +463,13 @@ subroutine interpo_search_2d( x, y, pointx, pointy, i, j, undeff )
   real, intent(in) :: pointy  !  y
   integer, intent(inout) :: i  ! pointx ͤۤʤֹͤ
   integer, intent(inout) :: j  ! pointy ͤۤʤֹͤ
-  real, intent(in), optional :: undeff  ! õϰϤǤ꾮ͤõ褦Ȥ, undef ֤,  undef ͤꤹ. default Ǥ 0.
+  integer, intent(in), optional :: undeff  ! õϰϤǤ꾮ͤõ褦Ȥ, undef ֤,  undef ͤꤹ. default Ǥ 0.
   integer :: just
 
   if(present(undeff))then
      just=int(undeff)
-     call interpo_search_1d( x, pointx, i, real(just) )
-     call interpo_search_1d( y, pointy, j, real(just) )
+     call interpo_search_1d( x, pointx, i, just )
+     call interpo_search_1d( y, pointy, j, just )
   else
      call interpo_search_1d( x, pointx, i )
      call interpo_search_1d( y, pointy, j )
@@ -501,14 +491,14 @@ subroutine interpo_search_3d( x, y, z, pointx, pointy, pointz, i, j, k, undeff )
   integer, intent(inout) :: i  ! pointx ͤۤʤֹͤ
   integer, intent(inout) :: j  ! pointy ͤۤʤֹͤ
   integer, intent(inout) :: k  ! pointz ͤۤʤֹͤ
-  real, intent(in), optional :: undeff  ! õϰϤǤ꾮ͤõ褦Ȥ, undef ֤,  undef ͤꤹ. default Ǥ 0.
+  integer, intent(in), optional :: undeff  ! õϰϤǤ꾮ͤõ褦Ȥ, undef ֤,  undef ͤꤹ. default Ǥ 0.
   integer :: just
 
   if(present(undeff))then
      just=int(undeff)
-     call interpo_search_1d( x, pointx, i, real(just) )
-     call interpo_search_1d( y, pointy, j, real(just) )
-     call interpo_search_1d( z, pointz, k, real(just) )
+     call interpo_search_1d( x, pointx, i, just )
+     call interpo_search_1d( y, pointy, j, just )
+     call interpo_search_1d( z, pointz, k, just )
   else
      call interpo_search_1d( x, pointx, i )
      call interpo_search_1d( y, pointy, j )
@@ -518,16 +508,23 @@ subroutine interpo_search_3d( x, y, z, pointx, pointy, pointz, i, j, k, undeff )
 end subroutine interpo_search_3d
 
 
-subroutine interpolation_1d( tmin, tmax, xmin, xmax, point, val )
+subroutine interpolation_1d( x, y, point, val )
   ! 1 ޥ롼
   implicit none
-  real, intent(in) :: tmin  ! κü
-  real, intent(in) :: tmax  ! αü
-  real, intent(in) :: xmin  ! tmin Ǥ
-  real, intent(in) :: xmax  ! tmax Ǥ
+  real, intent(in) :: x(2)  ! κü
+  real, intent(in) :: y(2)  ! x Ƥ
   real, intent(in) :: point  ! 
   real, intent(inout) :: val  ! Ǥ
   real :: fd, dt
+  real :: tmin
+  real :: tmax
+  real :: xmin
+  real :: xmax
+
+  tmin=x(1)
+  tmax=x(2)
+  xmin=y(1)
+  xmax=y(2)
 
   dt=point-tmin
 
@@ -550,13 +547,13 @@ subroutine interpolation_2d( x, y, z, point, val )
   real :: valx(2)
 
   ! y(1) Ǥ x Ǥ
-  call interpolation_1d( x(1), x(2), z(1,1), z(2,1), point(1), valx(1) )
+  call interpolation_1d( (/x(1), x(2)/), (/z(1,1), z(2,1)/), point(1), valx(1) )
 
   ! y(2) Ǥ x Ǥ
-  call interpolation_1d( x(1), x(2), z(1,2), z(2,2), point(1), valx(2) )
+  call interpolation_1d( (/x(1), x(2)/), (/z(1,2), z(2,2)/), point(1), valx(2) )
 
   ! x  y Ǥ(줬)
-  call interpolation_1d( y(1), y(2), valx(1), valx(2), point(2), val )
+  call interpolation_1d( (/y(1), y(2)/), (/valx(1), valx(2)/), point(2), val )
 
 end subroutine interpolation_2d
 
@@ -580,7 +577,7 @@ subroutine interpolation_3d( x, y, z, u, point, val )
   call interpolation_2d( x, y, u(:,:,2), point(1:2), valx(2) )
 
   ! z(1)  z Ǥ(줬)
-  call interpolation_1d( z(1), z(2), valx(1), valx(2), point(3), val )
+  call interpolation_1d( (/z(1), z(2)/), (/valx(1), valx(2)/), point(3), val )
 
 end subroutine interpolation_3d
 
@@ -629,7 +626,7 @@ subroutine LSM_poly( x, y, a, intercept, undef )
 ! LSM_poly Ǥ, F(x)=\sum^{N}_{n=0}{a_nx^n}
 ! Ǥռ¿༰ԤȤǽ.
 ! 르ꥺϺǾˡѤƤ, ΥФˤ gausss 롼.
-  use Algebra
+  use Matrix_Calc
   implicit none
   real, intent(in) :: x(:)  ! ǡ 1
   real, intent(in) :: y(size(x))  ! ǡ 2
@@ -761,6 +758,106 @@ subroutine Cor_Coe( x, y ,cc, error )  ! 2 ǡط׻롼
 
 end subroutine Cor_Coe
 
+!---------------------------------
+!---------------------------------
+
+subroutine Bubble_Sort_i( a, b, sig )
+! Х֥륽ȤѤƿͥǡ sig ˥Ȥ.
+  implicit none
+  integer, intent(in) :: a(:)  ! Ȥ
+  integer, intent(inout) :: b(size(a))  ! Ȥ̤Ǽ
+  character(1), intent(in) :: sig  ! Ȥν
+                                   ! 'i' = ֹμ㤤Τ˾ͤ
+                                   ! 'r' = ֹμ㤤Τ礭ͤ
+  integer :: i, j, k, n
+  integer :: tmp
+
+  n=size(a)
+
+  if(sig/='i'.and.sig/='r')then
+     write(*,*) "### ERROR ###"
+     write(*,*) "sig flag is 'r' .or. 'i', STOP."
+     stop
+  end if
+
+  do i=1,n
+     b(i)=a(i)
+  end do
+
+  if(sig=='i')then  ! ٤
+     do i=1,n
+        do j=1,n-1
+           if(b(j)>b(j+1))then
+              tmp=b(j+1)
+              b(j+1)=b(j)
+              b(j)=tmp
+           end if
+        end do
+     end do
+  else
+     do i=1,n
+        do j=1,n-1
+           if(b(j)<b(j+1))then
+              tmp=b(j+1)
+              b(j+1)=b(j)
+              b(j)=tmp
+           end if
+        end do
+     end do
+  end if
+
+end subroutine Bubble_Sort_i
+
+!---------------------------------
+!---------------------------------
+
+subroutine Bubble_Sort_f( a, b, sig )
+! Х֥륽ȤѤƿͥǡ sig ˥Ȥ.
+  implicit none
+  real, intent(in) :: a(:)  ! Ȥ
+  real, intent(inout) :: b(size(a))  ! Ȥ̤Ǽ
+  character(1), intent(in) :: sig  ! Ȥν
+                                   ! 'i' = ֹμ㤤Τ˾ͤ
+                                   ! 'r' = ֹμ㤤Τ礭ͤ
+  integer :: i, j, k, n
+  real :: tmp
+
+  n=size(a)
+
+  if(sig/='i'.and.sig/='r')then
+     write(*,*) "### ERROR ###"
+     write(*,*) "sig flag is 'r' .or. 'i', STOP."
+     stop
+  end if
+
+  do i=1,n
+     b(i)=a(i)
+  end do
+
+  if(sig=='i')then  ! ٤
+     do i=1,n
+        do j=1,n-1
+           if(b(j)>b(j+1))then
+              tmp=b(j+1)
+              b(j+1)=b(j)
+              b(j)=tmp
+           end if
+        end do
+     end do
+  else
+     do i=1,n
+        do j=1,n-1
+           if(b(j)<b(j+1))then
+              tmp=b(j+1)
+              b(j+1)=b(j)
+              b(j)=tmp
+           end if
+        end do
+     end do
+  end if
+
+end subroutine Bubble_Sort_f
+
 !---------------------------------
 !---------------------------------
 !---- ʲ, private 롼----
