@@ -3,7 +3,7 @@ module Thermo_Advanced_Routine
 use Thermo_Function
 use Thermo_Routine
 use Thermo_Advanced_Function
-use analy
+use Derivation
 
 contains
 
@@ -165,7 +165,7 @@ end subroutine
 subroutine EDC_TKE( x, y, z, pt, tke, nuth, nutv, nuhh, nuhv )
   use Phys_Const
   use Statistics
-  use Analy
+  use Derivation
 ! 1.5  TKE ѤǴ׻.
   real, intent(in) :: x(:)  ! x κɸѿ [m]
   real, intent(in) :: y(:)  ! y κɸѿ [m]
@@ -177,28 +177,49 @@ subroutine EDC_TKE( x, y, z, pt, tke, nuth, nutv, nuhh, nuhv )
   real, intent(inout) :: nuhh(size(x),size(y),size(z))  ! ʿȻ [m^2/s]
   real, intent(inout) :: nuhv(size(x),size(y),size(z))  ! ľȻ [m^2/s]
   real :: ptb(size(z)), BV(size(z))
-  real, dimension(size(x),size(y),size(z)) :: lv, ls
-  real :: lh, dsh, dsv
   real, parameter :: alpha=1.0e-6
   integer :: i, j, k
   integer :: nx  !  1 ǿ
   integer :: ny  !  2 ǿ
   integer :: nz  !  3 ǿ
-  real :: dx  ! x γʻҴֳ [m]
-  real :: dy  ! y γʻҴֳ [m]
-  real :: dz  ! z γʻҴֳ [m]
+  real :: dx(size(x))  ! x γʻҴֳ [m]
+  real :: dy(size(y))  ! y γʻҴֳ [m]
+  real :: dz(size(z))  ! z γʻҴֳ [m]
+  real :: dsh(size(x),size(y)), dsv(size(z)), lh(size(x),size(y))
+  real, dimension(size(x),size(y),size(z)) :: lv, ls
   intrinsic :: min, max
 
-  dx=x(2)-x(1)
-  dy=y(2)-y(1)
-  dz=z(2)-z(1)
   nx=size(x)
   ny=size(y)
   nz=size(z)
 
-  dsh=sqrt(dx*dy)
-  dsv=dz
-  lh=dsh
+  do i=2,nx-1
+     dx(i)=0.5*(x(i+1)-x(i-1))
+  end do
+  do j=2,ny-1
+     dy(j)=0.5*(y(j+1)-y(j-1))
+  end do
+  do k=2,nz-1
+     dz(k)=0.5*(z(k+1)-z(k-1))
+  end do
+
+  dx(1)=x(2)-x(1)
+  dx(nx)=x(nx)-x(nx-1)
+  dy(1)=y(2)-y(1)
+  dy(ny)=y(ny)-y(ny-1)
+  dz(1)=z(2)-z(1)
+  dz(nz)=z(nz)-z(nz-1)
+
+  do j=1,ny
+     do i=1,nx
+        dsh(i,j)=sqrt(dx(i)*dy(j))
+        lh(i,j)=dsh(i,j)
+     end do
+  end do
+
+  do k=1,nz
+     dsv(k)=dz(k)
+  end do
 
 ! ̤οʿʿ
   do i=1,nz
@@ -214,14 +235,14 @@ subroutine EDC_TKE( x, y, z, pt, tke, nuth, nutv, nuhh, nuhv )
         do i=1,nx
            if(BV(k)>0.0)then
               ls(i,j,k)=0.76*sqrt(tke(i,j,k)/BV(k))
-              lv(i,j,k)=min(dsv,ls(i,j,k))
+              lv(i,j,k)=min(dsv(k),ls(i,j,k))
            else
-              lv(i,j,k)=dsv
+              lv(i,j,k)=dsv(k)
            end if
-           nuth(i,j,k)=max(0.1*sqrt(tke(i,j,k))*lh,alpha*(dsh**2))
-           nutv(i,j,k)=max(0.1*sqrt(tke(i,j,k))*lv(i,j,k),alpha*(dsv**2))
+           nuth(i,j,k)=max(0.1*sqrt(tke(i,j,k))*lh(i,j),alpha*(dsh(i,j)**2))
+           nutv(i,j,k)=max(0.1*sqrt(tke(i,j,k))*lv(i,j,k),alpha*(dsv(k)**2))
            nuhh(i,j,k)=3.0*nuth(i,j,k)
-           nuhv(i,j,k)=nutv(i,j,k)*(1.0+2.0*(lv(i,j,k)/dsv))
+           nuhv(i,j,k)=nutv(i,j,k)*(1.0+2.0*(lv(i,j,k)/dsv(k)))
         end do
      end do
   end do
@@ -233,7 +254,7 @@ end subroutine
 
 subroutine Brunt_Freq( x, y, z, pt, BV, undeff )
 ! ֥ȥХ鿶ư 2 ׻.
-  use analy
+  use Derivation
   use Phys_Const
   implicit none
   real, intent(in) :: x(:)  ! x κɸѿ [m]
@@ -302,11 +323,11 @@ end subroutine
 !--------------------------------------------------------
 !--------------------------------------------------------
 
-subroutine Ertel_PV( x, y, z, u, v, w, rho, pt, cor, pv, undeff, hx, hy, hz )
+subroutine Ertel_PV( x, y, z, u, v, w, rho, pt, cor, pv, undeff, sx, sy, sz )
 ! ƥΥݥƥ󥷥뱲٤׻
   use Thermo_Function
   use Thermo_Routine
-  use analy 
+  use derivation
   implicit none
   real, intent(in) :: x(:)  ! x κɸѿ [m]
   real, intent(in) :: y(:)  ! y κɸѿ [m]
@@ -319,9 +340,9 @@ subroutine Ertel_PV( x, y, z, u, v, w, rho, pt, cor, pv, undeff, hx, hy, hz )
   real, intent(in) :: cor(size(x),size(y))  ! ꥪѥ᡼ [/s]
   real, intent(inout) :: pv(size(x),size(y),size(z))  ! PV [Km^2/kgs]
   real, intent(in), optional :: undeff
-  real, intent(in), optional :: hx(size(x),size(y),size(z))  ! 󥰥ե
-  real, intent(in), optional :: hy(size(x),size(y),size(z))  ! 󥰥ե
-  real, intent(in), optional :: hz(size(x),size(y),size(z))  ! 󥰥ե
+  real, intent(in), optional :: sx(size(x),size(y),size(z))  ! 󥰥ե
+  real, intent(in), optional :: sy(size(x),size(y),size(z))  ! 󥰥ե
+  real, intent(in), optional :: sz(size(x),size(y),size(z))  ! 󥰥ե
   real :: tmp1(size(x),size(y),size(z))
   real :: tmp2(size(x),size(y),size(z))
   real :: tmp3(size(x),size(y),size(z))
@@ -341,13 +362,13 @@ subroutine Ertel_PV( x, y, z, u, v, w, rho, pt, cor, pv, undeff, hx, hy, hz )
   ny=size(y)
   nz=size(z)
 
-  if(present(hx).and.present(hy).and.present(hz))then
+  if(present(sx).and.present(sy).and.present(sz))then
      do k=1,nz
         do j=1,ny
            do i=1,nx
-              scalex(i,j,k)=hx(i,j,k)
-              scaley(i,j,k)=hy(i,j,k)
-              scalez(i,j,k)=hz(i,j,k)
+              scalex(i,j,k)=sx(i,j,k)
+              scaley(i,j,k)=sy(i,j,k)
+              scalez(i,j,k)=sz(i,j,k)
            end do
         end do
      end do
@@ -365,19 +386,19 @@ subroutine Ertel_PV( x, y, z, u, v, w, rho, pt, cor, pv, undeff, hx, hy, hz )
 
   if(present(undeff))then
 !  ̤ζָۤ׻.
-     call grad_vec_3d( x, y, z, pt, tmp1, tmp2, tmp3, undeff,  &
+     call grad_3d( x, y, z, pt, tmp1, tmp2, tmp3, undeff,  &
   &                    scalex, scaley, scalez )
 !  3  rotation ׻.
-     call rotate( x, y, z, u, v, w, tmp4, tmp5, tmp6, undeff,  & 
+     call curl_3d( x, y, z, u, v, w, tmp4, tmp5, tmp6, undeff,  & 
   &               scalex, scaley, scalez )
 !  omega  grad pt Ѥ׻
      call dot_prod( tmp4, tmp5, tmp6, tmp1, tmp2, tmp3, tmp7, undeff )
   else
 !  ̤ζָۤ׻.
-     call grad_vec_3d( x, y, z, pt, tmp1, tmp2, tmp3,  &
+     call grad_3d( x, y, z, pt, tmp1, tmp2, tmp3,  &
   &                    hx=scalex, hy=scaley, hz=scalez )
 !  3  rotation ׻.
-     call rotate( x, y, z, u, v, w, tmp4, tmp5, tmp6,  & 
+     call curl_3d( x, y, z, u, v, w, tmp4, tmp5, tmp6,  & 
   &               hx=scalex, hy=scaley, hz=scalez )
 !  omega  grad pt Ѥ׻
      call dot_prod( tmp4, tmp5, tmp6, tmp1, tmp2, tmp3, tmp7 )
@@ -425,6 +446,40 @@ end subroutine
 
 !--------------------------------------------------------
 !--------------------------------------------------------
+!!!!!!!!!!!!!!!!!!!1 under construction
+subroutine Buoyanc( rhop, rhob, buo, qall )
+  use phys_const
+  implicit none
+  real, intent(in) :: rhop(:,:,:)
+  real, intent(in) :: rhob(size(rhop,3))
+  real, intent(inout) :: buo(size(rhop,1),size(rhop,2),size(rhob))
+  real, intent(in), optional :: qall(size(rhop,1),size(rhop,2),size(rhob))
+  integer :: nx, ny, nz
+  integer :: i, j, k
+
+  nx=size(rhop,1)
+  ny=size(rhop,2)
+  nz=size(rhop,3)
+
+  if(present(qall))then
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              buo(i,j,k)=g*(rhop(i,j,k)-rhob(k))/(rhob(k))
+           end do
+        end do
+     end do
+  else
+     do k=1,nz
+        do j=1,ny
+           do i=1,nx
+              buo(i,j,k)=g*(rhop(i,j,k)-rhob(k))/(rhob(k))
+           end do
+        end do
+     end do
+  end if
+
+end subroutine Buoyanc
 
 !--------------------------------------------------------
 !--------------------------------------------------------
